
import numpy
import sys



import csdp


print 'Loading problem and solution....'
problem = csdp.read_problem_file("test.7.dat-s")
solution = csdp.read_solution_file('test.7.result.txt',problem)


def round_one_number(x):
    denominator=10000
    return QQ(floor(x*denominator))/QQ(denominator)

def round_vector(v):
    vQQ = vector(QQ,len(v))
    for i in range(len(v)): vQQ[i] = round_one_number(v[i])
    return vQQ

def round_block(block):
    eig = numpy.linalg.eigh(block)
    eigQQ=[round_vector(eig[0]),[]]
    for i in range(len(eig[1])):
        eigQQ[1].append(round_vector(eig[1][i]))

    Asize = len(eig[0])
    MS = MatrixSpace(QQ,Asize,Asize)
    U = MS()  # With eigebvectors
    D = MS()  # diagonal

    for i in range(Asize):
        D[i,i] = eigQQ[0][i]

    for i in range(Asize):
        for j in range(Asize):
            U[i,j] = eigQQ[1][j][i]

    return (U.transpose()*D*U)


print 'Rounding solution....'
# the last block are just slack variables
rounded = [ round_block(block) for block in solution['Primal'][:len(solution['Primal'])-1]]


blocks = problem['num_blocks'] - 1  # len(problem['blocks']) - 1 # the last block are slack variables

dim = []
dimsum = 0
variables = 0

for i in range(blocks):
    dim.append(problem['block_sizes'][i])
    variables += dim[i]*(dim[i]+1)/2
    dimsum += dim[i]



# Change matrix to a linear vector
def matrixToLinear(blocks,dim,M):
    variables = 0
    for b in range(blocks):
        variables += (dim[b]*(dim[b]+1))/2
    L=vector(RDF,variables)
    cnt = 0
    for b in range(blocks):
        for i in range(dim[b]):
            for j in range(i,dim[b]):
                L[cnt] = M[b][j,i]
                cnt=cnt+1
    return L

#load('checkSolution.py')

print 'Changing to linear form....'
L = matrixToLinear(blocks, dim, rounded)


# a bit more memory efficient thing
def checkVecSolution(problem, variables):
    num_constraints = problem['num_constraints']
    num_blocks = problem['num_blocks']
    block_sizes = problem['block_sizes']
    blocks = problem['blocks']
    constants = problem['constants']
    minsofar=QQ(1000)
    RoundingLinearSystemVariableNumber = len(variables)
    OneLineOriginalProbelmVector = vector(QQ, RoundingLinearSystemVariableNumber)
    ProductsLineVariables = vector(QQ, num_constraints)
    constantsVector = vector(QQ,constants)
    constantsVector = vector(QQ,constantsVector)
    for i in range(num_constraints):
        k = 0
        ProductsLineVariables[i] = 0
        for index in range(num_blocks):
            size = block_sizes[index]
            if size > 0:
                for x in range(size):
                    for y in range(x,size):
                        if x == y:
                            OneLineOriginalProbelmVector[k] = blocks[index][x,y,i+1]
                        else:
                            OneLineOriginalProbelmVector[k] = 2 * blocks[index][x,y,i+1]
                        k += 1
        ProductsLineVariables[i] = OneLineOriginalProbelmVector*variables
        difference = float(constantsVector[i]-float(ProductsLineVariables[i]))
        if minsofar > difference: minsofar = difference 
        print 'done {}/{} product {} and constant {} with diff {} and min {}'.format(i,num_constraints,float(ProductsLineVariables[i]),constantsVector[i],difference,minsofar)
    return minsofar
#    return [ProductsLineVariables, constantsVector]

print 'Verifying solution...'
minsofar=checkVecSolution(problem,L)
print 'Minimum is {}~{}'.format(minsofar,float(minsofar))
print 'Saving rounded solution...'

f = open('rounded.txt','w')
f.write(str(L))
f.close()

print 'Done.'
