/* 

g++ -Wall sdp_generator.cpp -O3 -fopenmp -o sdp_generator && ./sdp_generator > test.7.dat-s
 
 */

#include <iostream>
#include <fstream>
#include <sstream>
#include <istream>
#include <vector>
#include <utility>
#include <assert.h>
#include <cstring>
#include <algorithm>
#include <cstdarg>
#include <cmath>
#include <limits> 


using namespace std;

//#define UPPER_BOUND 
#define LOWER_BOUND   


#define SDP_PRINT_SPECIFIC_PART
#define SDP_PRINT_FLAG_PRODUCT



#define G_COLORED_VERTICES
#define G_CROSSINGS
#define G_BE_BRAVE_AND_IGNORE_SAFETY_ASSERTS
const int     Kn = 7;
#define V      9
#define COLORS_VERTICES 4 // tripartite


#define MAX_FLAG_TYPES  30000 // Just a constant - raise if neded - means different theta


#define WRITE_INTEGER_DENSITIES_FOR_FLAGS


class flag
{
public:
    flag()
    {
		m_Theta = 0;
		m_vertices = 0;
    }
    
    virtual ~flag()
    {
        
    }
    
    
    // First m_Theta vertices are considered labeled in that order
    void set_Theta(int Theta)
    {
		//cout << Theta << m_vertices << " ";
        assert(Theta < m_vertices);
        m_Theta = Theta;
    }
    
	
    int get_Theta()
    {
        return m_Theta;
    }
    
    void set_vertices_and_Theta(int vertices, int Theta)
    {
        set_vertices(vertices);
        set_Theta(Theta);
    }
	
	
    void set_vertices(int vertices)
    {
        assert(vertices <= V);
        m_vertices = vertices;

#ifdef G_COLORED_VERTICES
		for (int u = 0; u < V; u++)
		{
			m_color_vertex[u] = 0;
		}
		m_colored_vertices[0] = m_vertices;
		for (int c = 1; c < COLORS_VERTICES; c++) m_colored_vertices[c] = 0;
#endif

		
		
#ifdef G_CROSSINGS
		// now crossing of pairs of edges
		for (int u1 = 0; u1 < V; u1++)
			for (int u2 = 0; u2 < V; u2++)
				for (int v1 = 0; v1 < V; v1++)
					for (int v2 = 0; v2 < V; v2++)
						m_cross_edges[u1][u2][v1][v2] = 0;
		m_crossings_cnt = -1;
		m_crossings_in_type = 0;
#endif


        
    }

    void swap_colors(int color_a = 1, int color_b = 2)
    {


#ifdef G_COLORED_VERTICES
  		for (int u = 0; u < m_vertices; u++)
  		{
           if (m_color_vertex[u] == 1) m_color_vertex[u] = 0;
	       else if (m_color_vertex[u] == 0) m_color_vertex[u] = 1;
		}
#endif
    }

    
	
#ifdef G_COLORED_VERTICES
    void color_vertex(int u, int color)
    {
    	assert(u < m_vertices);
    	assert(color >= 0);
        //cerr << color << " on " << u <<  endl;
        assert(color < COLORS_VERTICES);
        
		m_colored_vertices[m_color_vertex[u]]--;
		
    	m_color_vertex[u] = color;

    	m_colored_vertices[color]++;
    }
#endif

	
	// perm is mapping vertices of h to vertices of this. pv is a candidate for
	// mapping of v, all vertices before v are already mapped.
	bool is_map_up_to_v_correct(int v, int pv, const int *perm, const flag &h) const
	{
			
#ifdef G_COLORED_VERTICES
		if (m_color_vertex[pv] != h.m_color_vertex[v]) return false;
#endif
			
	
#ifdef G_CROSSINGS
		// check crossings
		// now crossing of pairs of edge
		for (int u2 = 0; u2 < v; u2++)
		for (int v1 = 0; v1 < v; v1++)
		{
			if (v1 == u2) continue;
			for (int v2 = v1+1; v2 < v; v2++)
			{
				if (v2 == u2) continue;
				if (m_cross_edges[pv][perm[u2]][perm[v1]][perm[v2]] != h.m_cross_edges[v][u2][v1][v2]) return false;
			}
		}
#endif

		return true;
	}

    // Real isomorphism testing
	bool make_perm_isomorphic(int v, int *perm, bool *used, const flag &h) const
	{
		
		// Test if the permutation is isomorphism
		if (v >= m_vertices)
		{
#ifndef G_BE_BRAVE_AND_IGNORE_SAFETY_ASSERTS

#ifdef G_COLORED_VERTICES
			// same vertex color
			for (int u = 0; u < m_vertices; u++)
				assert(m_color_vertex[perm[u]] == h.m_color_vertex[u]);
#endif

#ifdef G_CROSSINGS
			// Check crossings
			// now crossing of pairs of edges
			for (int u1 = 0; u1 < m_vertices; u1++)
			for (int u2 = u1+1; u2 < m_vertices; u2++)
			for (int v1 = u1+1; v1 < m_vertices; v1++)
			for (int v2 = v1+1; v2 < m_vertices; v2++)
			{
				assert(m_cross_edges[perm[u1]][perm[u2]][perm[v1]][perm[v2]] == h.m_cross_edges[u1][u2][v1][v2]);
			}
#endif
		
#endif
			return true;
		}
		
		for (int pv = m_Theta; pv < m_vertices; pv++)
		{
			// check if used
			if (used[pv]) continue;

			if (!is_map_up_to_v_correct(v, pv, perm, h)) continue;

			perm[v] = pv;
			used[pv]  = true;
			
			if (make_perm_isomorphic(v+1, perm, used, h)) return true;
			
			used[pv]  = false;
		}
		
		return false;
	}
	
	
	bool is_isomorphic_to(const flag &h) const
	{
        // quick check
        if (m_vertices != h.m_vertices) return false;
        if (m_Theta != h.m_Theta) return false;
#ifdef G_CROSSINGS
        if (m_crossings_cnt != h.m_crossings_cnt) return false;
#endif
		
#ifdef G_CROSSINGS
		if (m_crossings_cnt!= -1 && h.m_crossings_cnt != -1 && m_crossings_cnt != h.m_crossings_cnt) return false;
#endif

#ifdef G_COLORED_VERTICES
        for (int c = 0; c < COLORS_VERTICES; c++)
        {
	        if (m_colored_vertices[c] != h.m_colored_vertices[c]) return false;
	    }
#endif
				
		// Check for same type
		if (!have_same_type(h)) return false;
		
		// try check all permutations --- WASTING HERE!!!!!!!!
		int perm[V];
		bool used[V];

		// Theta is preserved by the mapping
        for (int i = 0; i < m_Theta; i++)
        {
            perm[i] = i;
            used[i] = true;
        }
        
        for (int i = m_Theta; i < V; i++)
        {
            perm[i] = 0;
            used[i] = false;
        }
		
		return make_perm_isomorphic(m_Theta, perm, used, h);
	}




    
    void as_subflag(const flag &h,int *mapping, int vertices, int theta)
    {
        set_vertices_and_Theta(vertices,theta);
		
		
#ifdef G_COLORED_VERTICES
		// copy vertices
        for (int x = 0; x < m_vertices; x++)
			color_vertex(x, h.m_color_vertex[mapping[x]]);
#endif

#ifdef G_CROSSINGS
		// copy crosings
		for (int u1 = 0; u1 < m_vertices; u1++)
		for (int u2 = 0; u2 < m_vertices; u2++)
		for (int v1 = 0; v1 < m_vertices; v1++)
		for (int v2 = 0; v2 < m_vertices; v2++)
		{
			m_cross_edges[u1][u2][v1][v2] = h.m_cross_edges[mapping[u1]][mapping[u2]][mapping[v1]][mapping[v2]];
		}
		m_crossings_cnt = count_crossings();
#endif
    }
    

	string print(const string &delimeter=" ") const
	{
		stringstream ss;
		
		// Prints the number of vertices, zero as not type and then part of the diagonal matrix
		ss << m_vertices << delimeter << m_Theta << delimeter << delimeter;
#ifdef G_COLORED_VERTICES
		print_vertex_coloring(ss, delimeter);
		ss << delimeter;
#endif
		
#ifdef G_CROSSINGS
		print_crossings(ss, delimeter);
		ss << delimeter;
#endif

		
		return ss.str();
	}
    
#ifdef G_COLORED_VERTICES
	void print_vertex_coloring(ostream &ostr, const string &delimiter) const
	{
		for (int i = 0; i < m_vertices; i++)
		{
			ostr << delimiter << m_color_vertex[i];
		}
	}
#endif
	
#ifdef G_CROSSINGS
	void print_crossings(ostream &ostr, const string &delimiter) const
	{
		for (int u1 = 0; u1 < m_vertices; u1++)
		{
			for (int u2 = u1+1; u2 < m_vertices; u2++)
			{
				for (int v1 = u1+1; v1 < m_vertices; v1++)
				{
					for (int v2 = v1+1; v2 < m_vertices; v2++)
					{
						if (u2 == v1 || u2==v2)  continue;
						ostr << delimiter << m_cross_edges[u1][u2][v1][v2];
					}
				}
			}
		}
	}
#endif
	

/***************************************************************************************************************** Loading ****************/	
	void load_from_string(const char *str)
	{
		stringstream s(str);
		load_from_stream(s, -1, -1);
	}
		
	
   bool load_from_stream(istream &stream, int assumed_vertices, int assumed_theta)
   {
		int vertices, theta;
		stream >> vertices >> theta;
		if (!stream.good()) return false; 
		        
		assert(assumed_theta == -1 || theta == assumed_theta);
      assert(assumed_vertices == -1 || assumed_vertices == vertices);
		set_vertices_and_Theta(vertices, theta);
		
#ifdef G_COLORED_VERTICES		
		for (int i = 0; i < m_vertices; i++)
		{
			int color_v;
			stream >> color_v;
			color_vertex(i,color_v);
		}
#endif		
		

#ifdef G_CROSSINGS
		for (int u1 = 0; u1 < m_vertices; u1++)
		{
			for (int u2 = u1+1; u2 < m_vertices; u2++)
			{
				for (int v1 = u1+1; v1 < m_vertices; v1++)
				{
					for (int v2 = v1+1; v2 < m_vertices; v2++)
					{
						if (u2 == v1 || u2==v2)  continue;
												
						int is_crossing;
						stream >> is_crossing;
						if (is_crossing)
							set_line_intersection(u1,u2,v1,v2,is_crossing);
					}
				}
			}
		}
		
		m_crossings_cnt = count_crossings();
#endif


		return true;
	}
	
	
    bool have_same_type(const flag &f) const
    {
        if (m_Theta != f.m_Theta) return false;

		
#ifdef G_COLORED_VERTICES
		// same vertex color
		for (int u = 0; u < m_Theta; u++)
			if (m_color_vertex[u] != f.m_color_vertex[u]) return false;
#endif
	
#ifdef G_CROSSINGS
		// Check crossings
		// now crossing of pairs of edges
		for (int u1 = 0; u1 < m_Theta; u1++)
			for (int u2 = u1+1; u2 < m_Theta; u2++)
				for (int v1 = u1+1; v1 < m_Theta; v1++)
					for (int v2 = v1+1; v2 < m_Theta; v2++)
						if (m_cross_edges[u1][u2][v1][v2] != f.m_cross_edges[u1][u2][v1][v2]) return false;
#endif

        return true;
    }
	
#ifdef G_CROSSINGS
	void set_line_intersection(int u1, int u2, int v1, int v2, int crossing)
	{
		if (m_cross_edges[u1][u2][v1][v2] == crossing) return;
		
		m_cross_edges[u1][u2][v1][v2] = crossing;
		m_cross_edges[u1][u2][v2][v1] = crossing;
		m_cross_edges[u2][u1][v1][v2] = crossing;
		m_cross_edges[u2][u1][v2][v1] = crossing;
		
		m_cross_edges[v1][v2][u1][u2] = crossing;
		m_cross_edges[v1][v2][u2][u1] = crossing;
		m_cross_edges[v2][v1][u1][u2] = crossing;
		m_cross_edges[v2][v1][u2][u1] = crossing;
	
       // if (crossing)
       //                 cerr << "xxxx" << endl;
		if (m_color_vertex[u1] != m_color_vertex[u2] && m_color_vertex[v1] != m_color_vertex[v2])
		{
			//if (crossing == 1) m_crossings_cnt++;
			//else m_crossings_cnt--;
		}
	}
	
	int count_crossings()
	{
		int crossings_cnt = 0;
		m_crossings_in_type = 0;
		
			// now crossing of pairs of edges
			for (int u1 = 0; u1 < m_vertices; u1++)
			{
				for (int u2 = u1+1; u2 < m_vertices; u2++)
				{
					for (int v1 = u1+1; v1 < m_vertices; v1++)
					{
						for (int v2 = v1+1; v2 < m_vertices; v2++)
						{
#ifdef G_COLORED_VERTICES
							// only cross crossings
							if (m_color_vertex[u1] == m_color_vertex[u2] || m_color_vertex[v1] == m_color_vertex[v2]) continue;
#endif
							if (m_cross_edges[u1][u2][v1][v2] == 1)
							{
								crossings_cnt++;
								if (u1 < m_Theta && u2 < m_Theta && v1 < m_Theta && v2 < m_Theta)
								{
									m_crossings_in_type++;
								}
							}
						}
					}
				}
			}
        return crossings_cnt;
	}
#endif

	

    bool partial_load_from_stream(istream &stream, int assumed_vertices)
    {
		int vertices, theta;
		stream >> vertices >> theta;
		
		
		if (!stream.good()) return false;
        if (theta != 0)
        {
            cerr << "You are trying to load an unlabeled flag from a stream containing flags." << endl;
            exit(1);
        }
        
		assert(m_vertices >= vertices);
  	    assert(assumed_vertices == vertices);
                  
		set_vertices_and_Theta(m_vertices,m_Theta); // reset the flag
		
#ifdef G_COLORED_VERTICES
		for (int u = 0; u < vertices; u++)
		{
				int color;
				stream >> color;
				color_vertex(u,color);
		}
#endif
		
		return true;
	}
    
    
public:
		
	int m_vertices;     // number of vertices
   int m_Theta; // number of labeled vertices
 	
	int m_id; // id of the original flag
	
	
#ifdef G_COLORED_VERTICES
	int m_color_vertex[V]; // color of a vertex (for bipartite complete flags?)
	int m_colored_vertices[COLORS_VERTICES];
#endif

	
#ifdef G_CROSSINGS
	int m_cross_edges[V][V][V][V];
	int m_crossings_cnt;
	int m_crossings_in_type; // number of crossings that are only part of the type
#endif	    
	
};




flag g;

//vector<flag> colored_Kn;
vector<flag> colored_all[V];
vector<flag>  g_flags[MAX_FLAG_TYPES]; // flags to process using multiplication
int           g_flag_types;




int binomial(int n, int k) {
    int b = 1;
    for (int i = 0; i < k; ++i) {
        b *= (n - i);
        b /= (i + 1);
    }
    return b;
}

int factorial(int n) {
    int b = 1;
    for (int i = 2; i < n; i++) {
        b *= i;
    }
    return b;
}


// Rest of F2 is also piskced as a subset
void pick_F2_mapping(const flag &F1, const flag &F2, const flag &H, int *mapping_F2, int next_to_map,  bool *used_from_H, int &good_maps)
{
    if (next_to_map >= F2.m_vertices)
    {
        flag H_F2;
        H_F2.as_subflag(H, mapping_F2, F2.m_vertices, F2.m_Theta);
        if (!H_F2.is_isomorphic_to(F2)) return;
		
        good_maps++;
        
        return;
    }
    
    int min_vertex = 0;
    if (next_to_map > F2.m_Theta) min_vertex = mapping_F2[next_to_map-1]+1;
    
    for (int v = min_vertex; v < H.m_vertices; v++)
    {
        if (used_from_H[v]) continue;
        
        used_from_H[v] = true;
        mapping_F2[next_to_map] = v;
        pick_F2_mapping(F1,F2,H,mapping_F2,next_to_map+1,used_from_H,good_maps);
        used_from_H[v] = false;
    }
    
}

// Rest of F1 is picked only as a subset
void pick_F1_mapping(const flag &F1, const flag &F2, const flag &H, int *mapping_F1, int *mapping_F2, int next_to_map,  bool *used_from_H, int &good_maps)
{
    if (next_to_map >= F1.m_vertices)
    {
		//cout << "." << endl;

        // Check if the mapping is corect!
        flag H_F1;
        H_F1.as_subflag(H, mapping_F1, F1.m_vertices,F1.m_Theta);
		
        if (!H_F1.is_isomorphic_to(F1)) return;
        
        pick_F2_mapping(F1,F2,H,mapping_F2,F2.m_Theta,used_from_H,good_maps);
        return;
    }
    
    int min_vertex = 0;
    if (next_to_map > F1.m_Theta) min_vertex = mapping_F1[next_to_map-1]+1;
    
    for (int v = min_vertex; v < H.m_vertices; v++)
    {
        if (used_from_H[v]) continue;
        
        used_from_H[v] = true;
        mapping_F1[next_to_map] = v;
        pick_F1_mapping(F1,F2,H,mapping_F1,mapping_F2,next_to_map+1,used_from_H,good_maps);
        used_from_H[v] = false;
    }
}

// Theta mapping is picked as all permutations...
void pick_theta_mapping(const flag &F1, const flag &F2, const flag &H, int *mapping_theta, int next_to_map, bool *used_from_H, int &good_maps)
{
    if (next_to_map >= F1.m_Theta)
    {
        int mapping_F1[F1.m_vertices];
        int mapping_F2[F2.m_vertices];
        for (int i = 0; i < F1.m_Theta; i++)
        {
            mapping_F2[i] = mapping_F1[i] = mapping_theta[i];
        }
				
        pick_F1_mapping(F1,F2,H,mapping_F1,mapping_F2, F1.m_Theta, used_from_H,good_maps);
        return;
    }
	
    for (int v = 0; v < H.m_vertices; v++)
    {
        if (used_from_H[v]) continue;
        
        if (!H.is_map_up_to_v_correct(next_to_map, v, mapping_theta, F1)) continue;
       
         used_from_H[v] = true;
         mapping_theta[next_to_map] = v;
         pick_theta_mapping(F1,F2,H,mapping_theta,next_to_map+1,used_from_H,good_maps);
		 used_from_H[v] = false;
    }
}



inline double P_F1_F2_IN_H(const flag &F1, const flag &F2, const flag &H)
{
    assert(F1.m_Theta == F2.m_Theta);

#ifdef G_COLORED_VERTICES
	for (int c = 1; c < COLORS_VERTICES; c++)
	{
		
		//cout << "v " << F1.m_colored_vertices[c] << " " << H.m_colored_vertices[c] << endl;
		if (F1.m_colored_vertices[c] > H.m_colored_vertices[c]) return 0;
		if (F2.m_colored_vertices[c] > H.m_colored_vertices[c]) return 0;
	}
#endif
	
#ifdef G_CROSSINGS	
	 if (F1.m_crossings_cnt + F2.m_crossings_cnt - F1.m_crossings_in_type > H.m_crossings_cnt) return 0;
#endif
		
    int mapping_theta[F1.m_Theta];
    bool used_from_H[H.m_vertices];
    
    for (int i = 0; i < H.m_vertices; i++)
    {
        used_from_H[i] = false;
    }
	
    int good_maps = 0;
    pick_theta_mapping(F1,F2,H,mapping_theta,0,used_from_H,good_maps);
	
    
    int all_maps = 1;
#ifndef WRITE_INTEGER_DENSITIES_FOR_FLAGS
    for (int i = 0; i < F1.m_Theta; i++) all_maps *= (H.m_vertices-i);
    all_maps *= binomial(H.m_vertices-F1.m_Theta,F1.m_vertices-F1.m_Theta);
    all_maps *= binomial(H.m_vertices-F1.m_vertices,F2.m_vertices-F1.m_Theta);
#endif
        
    return (double)good_maps/(double)all_maps;
}






double compute_density_for_SDP(const flag &g)
{
	return g.m_crossings_cnt;
}

// According to http://plato.asu.edu/ftp/sdpa_format.txt
//
// Solved program
//   max C.X
//   subject to A_1.X = b_1
//              A_2.X = b_2
//              A_mdim.X = b_mdim
//   where X is positive semidefinite matrix
//
// In our UPPER BOUND application
//
//      min    t
//      s.t.  D_i + [A.X]_i <= t
//
// We add a slack variable s_i to make the <= to = and move D_i (density - constant to the other side)
// and change to maximization:
//
//      max   -t
//      s.t.  [A.X]_i + s_i - t = -D_i
//
//   C.X is just entry 0 2 1 1 -1.0 which means that in the second block,
//   the first variable is with -1. Se we are maximizing -d which corresponds to minimizing d
//
// Now LOWER BOUND application
//
//      max    t
//      s.t.  D_i - [A.X]_i  >= t
//
// We add a slack variable s_i to make the >= to = and move D_i (density - constant to the other side)
// and change to maximization:
//
//      max   t
//      s.t.  [A.X]_i + s_i + t = D_i
//
// Finally, Additional constraints applications:
// Say we want to add  G >= c. Then we could add
//  k(G(sampled in H_i) -c) >= 0
//
// For lower bound we get
//     [A.X]_i + k*(G_i-c) + s_i + t = D_i
//
//
void print_CSDP()
{
    int mdim = 0;
    mdim = colored_all[Kn].size();
    
    
#ifdef SDP_PRINT_SPECIFIC_PART
    cout << mdim << endl; //<<" =mdim" << endl; // number of constraint matrices
    for (int f = 0; f < g_flag_types; f++) cout <<  g_flags[f].size() << " ";
    cout << -1-mdim << " ";
	cout << endl; // 2x2 and slack variable for every matrix and the - means that it is a diagonal matrix
    
    for (unsigned int i = 0; i < colored_all[Kn].size(); i++)
    {
#ifdef UPPER_BOUND
		cout << -1*compute_density_for_SDP(colored_all[Kn][i]) << " "; // outputing b_i
#endif
#ifdef LOWER_BOUND
		cout << compute_density_for_SDP(colored_all[Kn][i]) << " "; // outputing b_i
#endif
    }
    cout << endl;

#ifdef UPPER_BOUND
    cout << "0 " << g_flag_types+1 << " 1 1 -1.0" << endl; // Objective function
#endif
#ifdef LOWER_BOUND
    cout << "0 " << g_flag_types+1 << " 1 1 1.0" << endl; // Objective function
#endif
#endif // #ifdef SDP_PRINT_SPECIFIC_PART

    // printing of subflags
#pragma omp parallel for ordered schedule(dynamic)
        for (int i = 0; i < (int)colored_all[Kn].size(); i++)
        {
            cerr << i << endl;
            stringstream ss;
            
            int matrixID = i + 1;

#ifdef SDP_PRINT_FLAG_PRODUCT
            for (int f = 0; f < g_flag_types; f++)
            {
                for (unsigned int x = 0; x < g_flags[f].size(); x++)
                for (unsigned int y = x; y < g_flags[f].size(); y++)
                {
                    
                    double PF = P_F1_F2_IN_H(g_flags[f][x], g_flags[f][y], colored_all[Kn][i]);
                    if (PF != 0)
                    {
                        ss << matrixID <<  " " << f+1 << " " << x+1 << " " << y+1 << " " <<  PF << endl;
                    }
                }
            }
#endif // SDP_PRINT_FLAG_PRODUCT
         
#ifdef SDP_PRINT_SPECIFIC_PART                                   
#ifdef UPPER_BOUND
            ss << matrixID << " " << g_flag_types+1 << " 1 1 -1.0" << endl;
#endif
#ifdef LOWER_BOUND
            ss << matrixID << " " << g_flag_types+1 << " 1 1 1.0" << endl;
#endif            
            ss << matrixID << " " << g_flag_types+1 << " " << matrixID+1 << " " <<  matrixID+1 << " 1.0" << endl;
			

#endif			
#pragma omp ordered
{
            cout << ss.str();
}
        }
    
}





int get_flag_type_in_list(const flag& f)
{
	// Find correct type
	for (int i = 0; i < g_flag_types; i++)
	{
		if (g_flags[i].size() == 0)
		{
			cerr << "Some bug" << endl;
		}
		if (f.have_same_type(g_flags[i][0])) return i;
	}
	
	return -1;
}

inline void include_flag_in_list(const flag& f)
{
	int id = get_flag_type_in_list(f);
	if (id == -1)
	{
		assert(MAX_FLAG_TYPES >= g_flag_types+1);
		id = g_flag_types++;
	}
	g_flags[id].push_back(f);
}


inline void include_flag_in_list_if_new(const flag& f)
{
	int id = get_flag_type_in_list(f);
	if (id == -1)
	{
		include_flag_in_list(f);
		return;
	}
	
	bool new_flag = true;
	for (int i = 0; i < (int)g_flags[id].size(); ++i)
	{
		if (f.is_isomorphic_to(g_flags[id][i]))
		{
			new_flag = false;
			break;
		}
	}
	
	if (new_flag)
	{
		include_flag_in_list(f);
	}
}


string filename_prefix(int sizeKn)
{
	stringstream filename;
	
	filename << "F_" << sizeKn
#ifdef G_COLORED_VERTICES
	<< "_vertices" << COLORS_VERTICES-1
#endif
#ifdef G_CROSSINGS
	<< "_crossings"
#endif
    ;
	return filename.str();
}



void load_unlabeled_flags_from_file(int sizeKn)
{
	stringstream filename;

	filename << filename_prefix(sizeKn) << ".txt";
	
    ifstream infile;
    infile.open (filename.str().c_str(), ifstream::in);
    if (!infile.good())
    {
        cerr << "Failed opening file " << filename.str() << endl;
        exit(1);
    }
	
	cerr << "Loading file " << filename.str() << endl;
	
	while (g.load_from_stream(infile,sizeKn,0))
	{
		if (sizeKn != g.m_vertices)
		{
			cout << sizeKn << " " << g.m_vertices << endl;
		}
		assert(sizeKn == g.m_vertices);
#ifndef G_BE_BRAVE_AND_IGNORE_SAFETY_ASSERTS
		bool new_flag = true;
		for (int i = 0; i < (int)colored_all[sizeKn].size();i++)
		{
			if (g.is_isomorphic_to_colorblind(colored_all[sizeKn][i]))
			{
				new_flag = false;
			}
		}
		if (new_flag)
#endif
		{
			colored_all[sizeKn].push_back(g);
		}
	}
	
	infile.close();
}




void generate_flags_as_subflags_k7_3_2_2()
{
    flag F;
	int mapping[6];
    
    
	// (3, 1, 1) theta (3,0,0)
	for (int i = 0; i < (int)colored_all[Kn].size();i++)
	{
		mapping[0] = 0;
		mapping[1] = 1;
		mapping[2] = 2;
		
		do {
            
			for (int a = 3; a < 5; a++)
				for (int b = 5; b < 7; b++)
                    {
                        mapping[3] = a;
                        mapping[4] = b;
                        
                        F.as_subflag(colored_all[Kn][i], mapping, 5, 3); // taking a subflag
                        
                        include_flag_in_list_if_new(F);
                    }
			
		} while ( std::next_permutation(mapping,mapping+3) );
	}
    
	// (3, 2, 1) theta (3,2,0)
	for (int i = 0; i < (int)colored_all[Kn].size();i++)
	{
		mapping[0] = 0;
		mapping[1] = 1;
		mapping[2] = 2;
		
		do {
            
			for (int a = 3; a < 5; a++)
				for (int b = 3; b < 5; b++)
                    for (int c = 5; c < 7; c++)
                {
                    mapping[3] = a;
                    mapping[4] = b;
                    mapping[5] = c;
                    
                    F.as_subflag(colored_all[Kn][i], mapping, 6, 5); // taking a subflag
                    
                    include_flag_in_list_if_new(F);
                }
			
		} while ( std::next_permutation(mapping,mapping+3) );
	}
    
	// (3, 1, 2) theta (3,1,2)
	for (int i = 0; i < (int)colored_all[Kn].size();i++)
	{
		mapping[0] = 0;
		mapping[1] = 1;
		mapping[2] = 2;
		
		do {
            
			for (int a = 3; a < 5; a++)
				for (int b = 5; b < 7; b++)
                    for (int c = 5; c < 7; c++)
                    {
                        mapping[3] = c;
                        mapping[4] = b;
                        mapping[5] = a;
                        
                        F.as_subflag(colored_all[Kn][i], mapping, 6, 5); // taking a subflag
                        
                        include_flag_in_list_if_new(F);
                    }
			
		} while ( std::next_permutation(mapping,mapping+3) );
	}
    
    
	// (2,1,1) theta (1,0,0)
	for (int i = 0; i < (int)colored_all[Kn].size();i++)
	{
//		do {
			for (int x = 0; x < 3; x++)
				for (int y = 0; y < 3; y++)
            
			for (int a = 3; a < 5; a++)
				for (int b = 5; b < 7; b++)
                {
                    if (x == y) continue;
                    mapping[0] = x;
                    mapping[1] = y;
                    mapping[2] = a;
                    mapping[3] = b;
                    
                    F.as_subflag(colored_all[Kn][i], mapping, 4, 1); // taking a subflag
                    
                    include_flag_in_list_if_new(F);
                }
			
//		} while ( std::next_permutation(mapping,mapping+3) );
	}
    
	// (2,2,1) theta (1,2,0)
	for (int i = 0; i < (int)colored_all[Kn].size();i++)
	{
        //		do {
        for (int x = 0; x < 3; x++)
            for (int y = 0; y < 3; y++)
                for (int z = 3; z < 5; z++)
                for (int a = 3; a < 5; a++)
                    for (int b = 5; b < 7; b++)
                    {
                        if (x == y) continue;
                        mapping[0] = x;
                        mapping[1] = z;
                        mapping[2] = a;
                        mapping[3] = b;
                        mapping[4] = z;
                        
                        F.as_subflag(colored_all[Kn][i], mapping, 5, 3); // taking a subflag
                        
                        include_flag_in_list_if_new(F);
                    }
        
        //		} while ( std::next_permutation(mapping,mapping+3) );
	}
    
    // (2,1,2) theta (1,0,2)
    for (int i = 0; i < (int)colored_all[Kn].size();i++)
    {
        //		do {
        for (int x = 0; x < 3; x++)
            for (int y = 0; y < 3; y++)
                for (int z = 3; z < 5; z++)
                    for (int a = 5; a < 7; a++)
                        for (int b = 5; b < 7; b++)
                        {
                            if (x == y) continue;
                            mapping[0] = x;
                            mapping[1] = a;
                            mapping[2] = b;
                            mapping[3] = y;
                            mapping[4] = z;
                            
                            F.as_subflag(colored_all[Kn][i], mapping, 5, 3); // taking a subflag
                            
                            include_flag_in_list_if_new(F);
                        }
        
        //		} while ( std::next_permutation(mapping,mapping+3) );
    }
}



int main(int argc, char *argv[])
{
	load_unlabeled_flags_from_file(Kn);    // loads Kn
    	
	cerr << "Loaded "<< endl;

    generate_flags_as_subflags_k7_3_2_2();
	
	cerr << "got "<< g_flag_types << " types" << endl;
	
	assert(g_flag_types < MAX_FLAG_TYPES);
    
	print_CSDP();
	
	return 0;    
}

