/* 

 reads drawing of a complete graph and then it turns them into flags
 
 
 g++ texttoflags.cpp -Wall -O3 -fopenmp -o texttoflags && ./texttoflags < otypes.07.txt | sort | uniq >  F_7_vertices3_crossings.txt
 
 */

#include <iostream>
#include <fstream>
#include <sstream>
#include <istream>
#include <vector>
#include <utility>
#include <assert.h>
#include <cstring>
#include <algorithm>
#include <cstdarg>
#include <cmath>
#include <limits> 
#include <cstdio>

using namespace std;



#define PARTITES 3


// Returns true if the lines intersect, otherwise false.
bool is_line_intersection(float p0_x, float p0_y, float p1_x, float p1_y,
						   float p2_x, float p2_y, float p3_x, float p3_y)
{
    float s1_x, s1_y, s2_x, s2_y;
    s1_x = p1_x - p0_x;     s1_y = p1_y - p0_y;
    s2_x = p3_x - p2_x;     s2_y = p3_y - p2_y;
	
    float s, t;
    s = (-s1_y * (p0_x - p2_x) + s1_x * (p0_y - p2_y)) / (-s2_x * s1_y + s1_x * s2_y);
    t = ( s2_x * (p0_y - p2_y) - s2_y * (p0_x - p2_x)) / (-s2_x * s1_y + s1_x * s2_y);
	
    if (s >= 0 && s <= 1 && t >= 0 && t <= 1)
    {
        // Collision detected
        return true;
    }
	
    return false; // No collision
}

class Kn
{
	
public:
	
    // Does 3,2,2 partition
	bool balanced()
	{
		int c[PARTITES];
        for (int i = 0; i < PARTITES; i++) c[i] = 0;
		for (int u = 0; u < n; u++)
			c[color[u]]++;
		
        if (c[0] != 3) return false;
        if (c[1] != 2) return false;
		
		return true;
	}	
	
	
	void make_permuted(int perm[], Kn G)
	{
		id = G.id;
		n  = G.n;
		for (int i = 0; i < n; i++)
		{
			x[perm[i]] = G.x[i];
			y[perm[i]] = G.y[i];
			color[perm[i]] = G.color[i];
		}
	}
	
	
    // used to remove duplicates
	string find_lex_minimal()
	{
		int perm[10];
		for (int i = 0; i < 10; i++)perm[i] = i;
		
		std::sort (perm,perm+n);
					
		string lex_min = generate_print();
		
		Kn Gpermuted;
		do {						
			
				Gpermuted.make_permuted(perm, *this);
			
				string perm_string = Gpermuted.generate_print();
			
				if (perm_string.compare(lex_min) < 0) lex_min = perm_string;
            
		} while ( std::next_permutation(perm,perm+n) );
		
		return lex_min;
	}
	
	string generate_print()
	{
		stringstream ss;
		ss << n << "  0    ";

        for (int i = 0; i < n; i++)
        {
            ss << color[i] << " ";
        }
        ss << "   ";
		
		// now crossing of pairs of edges
		for (int u1 = 0; u1 < n; u1++)
		{
			for (int u2 = u1+1; u2 < n; u2++)
			{
				for (int v1 = u1+1; v1 < n; v1++)
				{
					for (int v2 = v1+1; v2 < n; v2++)
					{
						
						if (u2 == v1 || u2==v2)  continue;
						
						if (color[u1] == color[u2] || color[v1] == color[v2])
						{
							ss << "0 ";
							continue;
						}
                        
						if (is_line_intersection(x[u1],y[u1],x[u2],y[u2],x[v1],y[v1],x[v2],y[v2]))
							ss << "1 ";
						else
							ss << "0 ";
					}
				}
			}
		}
		
		ss << " ";
		

		return ss.str();
	}
	
    
	int n;
	int x[10];
	int y[10];
	int color[10];
	int id;
};



void make_coloring(Kn &G, stringstream &ss, int i)
{
	if (i >= G.n)
	{
        ss << G.find_lex_minimal() << endl;
        return;
	}
	
    for (int c = 0; c < PARTITES; c++)
    {
        G.color[i] = c;
        make_coloring(G, ss, i+1);
	}
}

int main(int argc, char *argv[])
{
    
    Kn G[1000];
    
    
	int n;
	
	cin >> n;
	
    int id = 0;
	while (cin)
	{
		for (int i = 0; i < n; i++)
		{
			cin >> G[id].x[i] >> G[id].y[i];
			G[id].x[i]++;
			G[id].y[i]++;
		}
		G[id].n = n;
        G[id].id = id+1;
        id++;
		
        if (!cin) break;

        if (id >= 1000)
        {
            cerr << "Increase the list for graphs!!!" << endl;
            return 0;
        }
    }
    
	cerr << "Loaded " << id << " graphs "<< endl;
    #pragma omp parallel for ordered schedule(dynamic)
    for (int i = 0; i < id; i++)
    {
        stringstream ss;
		make_coloring(G[i], ss, 0);
		
        #pragma omp ordered
        {
            cout << ss.str();
            cerr << G[i].id << endl;
        }
    }

	return 0;
}

